
<!--- 
	This action saves the given task. This can either be a new task
	or task (existing tasks will have an ID).
	
	POST:	/tasks/save
--->

<!--- 
	Param the FORM variables. Notice that we are giving the 
	parameters defaults; this is so we can give more granular 
	error messages.
--->
<cfparam name="form.id" type="string" default="" />
<cfparam name="form.task" type="string" default="" />
<cfparam name="form.dueDate" type="string" default="" />
<cfparam name="form.taskList" type="string" default="" />


<!--- ----------------------------------------------------- --->
<!--- ----------------------------------------------------- --->


<!--- Validate the post data. --->

<cfif !len( form.task )>
	
	<cfset arrayAppend(
		api.errors,
		"Please provide a task."
		) />

</cfif>

<cfif !isDate( form.dueDate )>
	
	<cfset arrayAppend(
		api.errors,
		"Please provide a valid due date."
		) />

</cfif>

<cfif !len( form.taskList )>
	
	<cfset arrayAppend(
		api.errors,
		"Please provide a task list under which to categorize this task."
		) />

<cfelseif reFind( "[^\w\s_-]", form.taskList )>

	<cfset arrayAppend(
		api.errors,
		"Task list names can only contain alpha-numeric characters, spaces, dashes, and underscores."
		) />
		
</cfif>


<!--- ----------------------------------------------------- --->
<!--- ----------------------------------------------------- --->


<!--- Check to see if there were any problems with the data. --->
<cfif arrayLen( api.errors )>

	<!--- 
		The user posted a bad request. Raise an malformed request
		exception. 
	--->
	<cfthrow type="BadRequest" />

</cfif>


<!--- ----------------------------------------------------- --->
<!--- ----------------------------------------------------- --->


<!--- 
	Now that we know we are working with valid data, let's make sure 
	the user is not accidentally double-posting this information.
	Let's compare this post to the previous post.
--->
<cflock
	name="session-#api.username#"
	type="exclusive"
	timeout="5">

	<!--- 
		Check to see if this is the same resource AND the same task 
		and taskList as the previous successful post.
		
		NOTE: We don't have to check for FORM keys since the previous 
		resource will indicate which form variables will be there.		
	--->
	<cfif (
		structKeyExists( api.session, "prevResource" ) &&
		(api.session.prevResource eq api.resource) &&
		(api.session.prevForm.task eq form.task) &&
		(api.session.prevForm.taskList eq form.taskList) &&
		(api.session.prevForm.dueDate eq form.dueDate)
		)>
		
		<!--- 
			Since this is a duplicate of the previous POST, let's 
			just return the previous data result.
		--->
		<cfset api.data = api.session.prevData />
		
		<!--- 
			Add a custom response header to indicate that the request
			as simply a re-response of the previous post. This is not
			required from a functional standpoint, but it's nice to 
			give the client as much information as possible.
			
			NOTE: Custom headers should begin with "X-" to indicate 
			that they are custom (not part of any HTTP specification).
		--->
		<cfset api.headers[ "X-Duplicate-Post" ] = api.resource />
		
		<!--- 
			Exit out of this processing branch (allowing the rest of 
			the API request to be executed with the previous data).
		--->
		<cfexit method="exitTemplate" />
		
	</cfif>
	
</cflock>
	
	
<!--- ----------------------------------------------------- --->
<!--- ----------------------------------------------------- --->


<!---
	Check to see if the user requested to update an actual task. 
	If so, we need to make sure the task exists.
--->
<cfif len( form.id )>
	
	<cfset task = application.taskService.getTask( form.id ) />
	
	<!--- 
		Check to see if we found a task AND that the task belonged 
		to the authorized user.
	--->
	<cfif !(
		task.recordCount &&
		(task.user eq api.username)	
		)>
	
		<!--- 
			The task either doesn't belong to the user or cannot be 
			found. Raise a not found exception.
		--->
		<cfthrow type="NotFound" />
		
	</cfif>
	
</cfif>


<!--- 
	Save the task data. At this point, we might be doing an update
	or an insert; either way, we should be good to go.
--->
<cfset taskID = application.taskService.saveTask(
	id = form.id,
	task = form.task,
	dueDate = form.dueDate,
	taskList = form.taskList,
	user = api.username
	) />

<!--- Return the task with ID as the response. --->
<cfset api.data = {
	id = taskID,
	task = form.task,
	dueDate = dateFormat( form.dueDate, "mmm d, yyyy" ),
	taskList = form.taskList
	} />
