
// Define the class.
;var TaskListManager = (function( $ ){

	// Define the class constructor.
	function Controller( username, password, baseAPI, target ){
		var self = this;
		
		// Store the username and password.
		this.username = username;
		this.password = password;
		
		// Store the root API URL. This will be the base of all the other 
		// API calls we make.
		this.baseAPI = baseAPI;
		
		// Store the UI aspect of this component.
		this.ui = target;
		
		// Store some of the key DOM elements for later use.
		this.dom = {};
		this.dom.form = this.ui.find( "#taskForm" );
		this.dom.taskLists = this.ui.find( "#taskLists" );
		this.dom.taskListTemplate = this.ui.find( "script.template.taskList" );
		this.dom.taskItemTemplate = this.ui.find( "script.template.taskItem" );
		
		
		// Bind the submit to the form so that we can save a task if necessary.
		this.dom.form.submit(
			function( event ){
				// Prevent the default event since we are routing this all through
				// the serer's API.
				event.preventDefault();
				
				// Save the task.
				self.saveTask({
					id: self.dom.form.find( "input[ name = 'id' ]" ).val(),
					task: self.dom.form.find( "input[ name = 'task' ]" ).val(),
					taskList: self.dom.form.find( "input[ name = 'taskList' ]" ).val(),
					dueDate: self.dom.form.find( "input[ name = 'dueDate' ]" ).val()
				});
			}
		);		
		
		
		// Bind the cancel form click.
		this.dom.form.find( "a.cancel" ).click(
			function( event ){
				// Prevent the default event.
				event.preventDefault();
				
				// Reset the form.
				self.clearForm();
			}
		);
		
		
		// Bind a click event listener to the list container so we can 
		// delegate clicking of the edit / delete events.
		this.dom.taskLists.click(
			function( event ){
				// Prevent the default event since there is nothing in the
				// list that should have a default behavior of note.
				event.preventDefault();
			
				// Get the target of the event.
				var target = $( event.target );
				
				// Check to see if the taret is an edit link.
				if (target.is( "a.edit" )){
				
					// Edit the task.
					self.editTask( target.closest( "li" ) );
				
				// Check to see if the target is a delete link.
				} else if (target.is( "a.delete" )){
				
					// Confirm that the user truly wants to delete the list item.
					if (confirm( "Delete this task?" )){
					
						// Delete the task.
						self.deleteTask( target.closest( "li" ) );
					
					}
				
				}
			}
		);
		
		
		// Pull the initial lists off the server and populate the page.
		this.loadLists();
	};
	
	
	// Define the class methods for this component.
	Controller.prototype = {
	
		// I call the API.
		api: function( method, resource, data, callback ){
			var self = this;
			
			// Make the AJAX request.
			$.ajax({
				// Before the API request gets made, we need to add the 
				// authorization headers for BASIC authentication.
				beforeSend: function( request ){
					request.setRequestHeader(
						"Authorization",
						("Basic " + Base64.encode( self.username + ":" + self.password ))
					);
				},
				
				// This approach *will* work, but it seems like the browser still
				// tries to use NTLM authentication first (before it uses this for
				// the basic authentication)??
				//
				// username: self.username,
				// password: self.password,
				
				// Turn off caching.
				cache: false,
				
				// The type of method (GET vs. POST).
				type: method,
				
				// Append the resource to the base API url. Notice that
				// we are adding the JSON format.
				url: (this.baseAPI + resource + ".json"),
				
				// Append any additional data.
				data: data,
				
				// All requests will be JSON.
				dataType: "json",
				
				// If the request is successful, execute the callback.
				success: function( response ){
					callback.call( self, response );
				},
				
				// Since we are using actual status codes from the API response,
				// let's check to see what status code was returned to see how
				// we need to respond. Remember, this is ONLY for errors.
				error: function( request ){
					var responseJSON = null;
				
					// Unauthorized.
					if (request.status === 401){
					
						// Parse the response text into error data.
						response = eval( "(" + request.responseText + ")" );
						
						// Alert errors.
						alert(
							"Please review the following:\n\n" + 
							response.join( "\n" )
						);						
					
					// Bad request (malformed request).
					} else if (request.status === 400){
						
						// Parse the response text into error data.
						response = eval( "(" + request.responseText + ")" );
					
						// Alert errors.
						alert(
							"Please review the following:\n\n" + 
							response.join( "\n" )
						);
					
					// Not acceptable (unsupported format requests).
					} else if (request.status === 406){
						
						// Parse the response text into error data.
						response = eval( "(" + request.responseText + ")" );
					
						// Alert errors.
						alert(
							"Please review the following:\n\n" + 
							response.join( "\n" )
						);
											
					// Not Found (resource not found).
					} else if (request.status === 404){
						
						// Parse the response text into error data.
						response = eval( "(" + request.responseText + ")" );
					
						// Alert errors.
						alert(
							"There was a problem with the given resource:\n\n" + 
							response.RESOURCE + "\n\n" +
							response.ERRORS.join( "\n" )
						);
											
					// Method Not Allowed (incorrect verb usage).
					} else if (request.status === 405){
						
						// Parse the response text into error data.
						response = eval( "(" + request.responseText + ")" );
					
						// Alert errors.
						alert(
							"There was a problem with the given resource:\n\n" + 
							response.RESOURCE + "\n\n" +
							response.ERRORS.join( "\n" )
						);
							
					// There was an unexpcted error.			
					} else {
					
						alert( "There was an unexpected error!" );						
						
					}
								
				}
			});			
		},
	
	
		// I populate the given template with the given data map and
		// return the result string.
		applyTemplate: function( template, map ){
			return(
				template.html().replace(
					new RegExp( "\\$\\{([^}]+)\\}", "g" ),
					function( $0, $1 ){
					
						// I this placeholder exists in the map, use 
						// it, otherwise, return the original match.
						if ($1 in map){
							return( map[ $1 ] );
						} else {
							return( $0 );
						}
						
					}
				)
			);
		},
		
		
		// I clear the form values.
		clearForm: function(){
			this.dom.form.find( "input[ name = 'id' ]" ).val( "" );
			this.dom.form.find( "input[ name = 'task' ]" ).val( "" );
			this.dom.form.find( "input[ name = 'taskList' ]" ).val( "" );
			this.dom.form.find( "input[ name = 'dueDate' ]" ).val( "" );
		},
		
		
		// I create a list with the given name. If the list already exists 
		// as part of the UI, I will just return the existing one.
		createList: function( name ){
			// Get the lists that have already been created.
			var existingLists = this.dom.taskLists.children();
			
			// Loop over the lists to see if we have an existing one.
			for ( var i = 0 ; i < existingLists.length ; i++){
				
				// Get the current list.
				var targetList = $( existingLists[ i ] );
				
				// Check to see if the data matches.
				if (targetList.data( "name" ) === name){
					
					// This is the list we are looking for, so just return the 
					// target list.
					return( targetList );
				
				}
			
			}
			
			// If we made it this far, we need to actually create the list.
			// Apply the name to the list template.
			var listHtml = this.applyTemplate(
				this.dom.taskListTemplate,
				{
					name: name
				}
			);
			
			// Create a list from the html.
			var targetList = $( listHtml );
			
			// Associate the name with the list.
			targetList.data( "name", name );
			
			// Append the list ot the UI.
			this.dom.taskLists.append( targetList );
			
			// Return the newly created list.
			return( targetList );
		},
		
		
		// I create the given task in the given list. If the task already exists, 
		// I simply return the existing reference.
		createTask: function( list, id, task, taskList, dueDate ){
			// Get the existing tasks. To start with, we are going to search ALL
			// tasks since we might be 
			var existingTasks = this.dom.taskLists.find( "li" );
			
			// Loop over the tasks to see if we have an existing one.
			for ( var i = 0 ; i < existingTasks.length ; i++){
				
				// Get the current task.
				var targetTask = $( existingTasks[ i ] );
				
				// Check to see if the data matches.
				if (targetTask.data( "id" ) == id){
				
					// Update the task wish the new data.
					targetTask.data({
						id: id,
						task: task,
						taskList: taskList,
						dueDate: dueDate					
					});	
					
					// Update the UI.
					targetTask.find( "span.dueDate" ).html( dueDate );
					targetTask.find( "span.task" ).html( task );		
					
					// Now that we have the task, check to see if it is in the 
					// right correct list. If it is not, we need to transfer it
					// to the correct list.
					var currentList = targetTask.closest( "div.taskList" );
					
					// Check to see if the task should be in the current list anymore.
					if (targetTask.data( "taskList" ) != currentList.data( "name" )){
					
						// The task being edited is now in the wrong list - let's
						// move it to the target list.
						targetTask.appendTo( list.find( "ul" ) );
						
						// Check to see if the old list is now empty.
						if (!currentList.find( "ul" ).children().size()){
						
							// Remove the list.
							currentList.remove();
						
						}
					
					}					
					
					// This id the task we are looking for, so just return the 
					// target task.
					return( targetTask );
				
				}
			
			}
			
			// If we made it this far, we need to actually create the task.
			// Apply the task data to the task template.
			var taskHtml = this.applyTemplate(
				this.dom.taskItemTemplate,
				{
					id: id,
					task: task,
					dueDate: dueDate
				}
			);
			
			// Create a task from the html.
			var targetTask = $( taskHtml );
			
			// Associate the task data with the task.
			targetTask.data({
				id: id,
				task: task,
				taskList: taskList,
				dueDate: dueDate
			});
			
			// Append the task ot the UI.
			list.find( "ul" ).append( targetTask );
			
			// Return the newly created task.
			return( targetTask );
		},
		
		
		// I delete the given task.
		deleteTask: function( task ){
			var self = this;
			var tasks = task.closest( "ul" );
			var list = tasks.closest( "div.taskList" );
			
			// Delete the task from the server.
			this.api(
				"post",
				"/tasks/delete",
				{
					id: task.data( "id" )
				},
				function( response ){
					// Remove the task from the UI.
					task.remove();
					
					// Check to see if there are any more tasks in this list.
					// If there are not, we can remove the entire list from
					// the UI (it can always be added back later with a new task).
					if (!tasks.children().size()){
						
						// Delete the list itself.
						list.remove();
						
					}
				}
			);
		},
		
		
		// I edit the task, moving the data into the form.
		editTask: function( task ){
			// Get the task data.
			var taskData = task.data();
			
			// Populate the form.
			this.dom.form.find( "input[ name = 'id' ]" ).val( taskData.id );
			this.dom.form.find( "input[ name = 'taskList' ]" ).val( taskData.taskList );
			this.dom.form.find( "input[ name = 'task' ]" ).val( taskData.task );
			this.dom.form.find( "input[ name = 'dueDate' ]" ).val( taskData.dueDate );
			
			// Focus the first input.
			this.dom.form.find( ":input:visible" ).focus();
		},
		
		
		// I load the lists based on the initial server request.
		loadLists: function(){
			var self = this;
			
			// Load the lists off of the server.
			this.api(
				"get",
				"/lists",
				{},
				function( response ){
					// Loop over the lists to create a new one for each 
					// data set.
					$.each(
						response,
						function( index, listData ){
							var list = self.createList( listData.NAME );
							
							// Load the tasks for this list.
							self.api(
								"get",
								("/lists/" + listData.NAME + "/tasks"),
								{},
								function( response ){
									
									// Loop over each task and create a task for it.
									$.each(
										response,
										function( index, taskData ){
											var task = self.createTask(
												list,
												taskData.ID,
												taskData.TASK,
												taskData.TASKLIST,
												taskData.DUEDATE
											);				
										}
									);		
																					
								}
							);		
									
						}
					);
				}
			);			
		},
		
		
		// I save the given task data.
		saveTask: function( taskData ){
			var self = this;
			
			// Save the task to the server.
			this.api(
				"post",
				"/tasks/save",
				taskData,
				function( response ){
					// Now that we have saved the task, let's crate a list for it 
					// (if one already exists, it'll just return that one).
					var list = this.createList( response.TASKLIST );
										
					// Create a task for this task data (if one already exists, it'll
					// just return that one).
					var task = this.createTask( list, response.ID, response.TASK, response.TASKLIST, response.DUEDATE );
					
					// Clear the form.
					this.clearForm();
					
					// If the original ID of the form task was empty, then populate the
					// form with the list so that the user can add mulitple tasks.
					if (!taskData.id.length){
						
						// Re-enter list name into form.
						self.dom.form.find( "input[ name = 'taskList' ]" ).val( taskData.taskList );
						
						// Focuse the task name.
						self.dom.form.find( "input[ name = 'task' ]" ).focus();
						
					}
				}
			);
		}

	};	

	
	// ------------------------------------------------------ //
	// ------------------------------------------------------ //
	
	// Return the class definition.
	return( Controller );	

})( jQuery );
